import 'dart:async';
import 'dart:convert';
import 'dart:ui';

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import 'menu_item.dart';
import 'tray_listener.dart';

const kEventOnTrayIconMouseDown = 'onTrayIconMouseDown';
const kEventOnTrayIconMouseUp = 'onTrayIconMouseUp';
const kEventOnTrayIconRightMouseDown = 'onTrayIconRightMouseDown';
const kEventOnTrayIconRightMouseUp = 'onTrayIconRightMouseUp';
const kEventOnTrayMenuItemClick = 'onTrayMenuItemClick';

class TrayManager {
  TrayManager._();

  /// The shared instance of [TrayManager].
  static final TrayManager instance = TrayManager._();

  final MethodChannel _channel = const MethodChannel('tray_manager');

  bool _inited = false;
  List<MenuItem> _menuItemList = [];

  ObserverList<TrayListener>? _listeners = ObserverList<TrayListener>();

  void _init() {
    _channel.setMethodCallHandler(_methodCallHandler);
    _inited = true;
  }

  Future<void> _methodCallHandler(MethodCall call) async {
    if (_listeners == null) return;

    final List<TrayListener> localListeners =
        List<TrayListener>.from(_listeners!);
    for (final TrayListener listener in localListeners) {
      if (_listeners!.contains(listener)) {
        switch (call.method) {
          case kEventOnTrayIconMouseDown:
            listener.onTrayIconMouseDown();
            break;
          case kEventOnTrayIconMouseUp:
            listener.onTrayIconMouseUp();
            break;
          case kEventOnTrayIconRightMouseDown:
            listener.onTrayIconRightMouseDown();
            break;
          case kEventOnTrayIconRightMouseUp:
            listener.onTrayIconRightMouseUp();
            break;
          case kEventOnTrayMenuItemClick:
            String identifier = call.arguments['identifier'];
            MenuItem menuItem = _menuItemList.firstWhere(
              (e) => e.identifier == identifier,
            );
            listener.onTrayMenuItemClick(menuItem);
            break;
        }
      }
    }
  }

  bool get hasListeners {
    return _listeners!.isNotEmpty;
  }

  void addListener(TrayListener listener) {
    _listeners!.add(listener);
  }

  void removeListener(TrayListener listener) {
    _listeners!.remove(listener);
  }

  Future<String?> get platformVersion async {
    final String? version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  Future<Rect> getBounds() async {
    final Map<dynamic, dynamic> resultData =
        await _channel.invokeMethod('getBounds');

    return Rect.fromLTWH(
      resultData['x'],
      resultData['y'],
      resultData['width'],
      resultData['height'],
    );
  }

  Future<void> setIcon(String icon) async {
    if (!_inited) this._init();

    ByteData imageData = await rootBundle.load(icon);
    String base64Icon = base64Encode(imageData.buffer.asUint8List());

    final Map<String, dynamic> arguments = {
      'base64Icon': base64Icon,
    };
    await _channel.invokeMethod('setIcon', arguments);
  }

  Future<void> setToolTip(String toolTip) async {
    final Map<String, dynamic> arguments = {
      'toolTip': toolTip,
    };
    await _channel.invokeMethod('setToolTip', arguments);
  }

  Future<void> setContextMenu(List<MenuItem> menuItems) async {
    _menuItemList = menuItems;
    final Map<String, dynamic> arguments = {
      'menuItems': menuItems.map((e) => e.toJson()).toList(),
    };
    await _channel.invokeMethod('setContextMenu', arguments);
  }

  Future<void> popUpContextMenu() async {
    await _channel.invokeMethod('popUpContextMenu');
  }
}
